/*******************************************************************************
* File Name: Pump2.c  
* Version 2.20
*
* Description:
*  This file contains APIs to set up the Pins component for low power modes.
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "cytypes.h"
#include "Pump2.h"

static Pump2_BACKUP_STRUCT  Pump2_backup = {0u, 0u, 0u};


/*******************************************************************************
* Function Name: Pump2_Sleep
****************************************************************************//**
*
* \brief Stores the pin configuration and prepares the pin for entering chip 
*  deep-sleep/hibernate modes. This function must be called for SIO and USBIO
*  pins. It is not essential if using GPIO or GPIO_OVT pins.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None 
*  
* \sideeffect
*  For SIO pins, this function configures the pin input threshold to CMOS and
*  drive level to Vddio. This is needed for SIO pins when in device 
*  deep-sleep/hibernate modes.
*
* \funcusage
*  \snippet Pump2_SUT.c usage_Pump2_Sleep_Wakeup
*******************************************************************************/
void Pump2_Sleep(void)
{
    #if defined(Pump2__PC)
        Pump2_backup.pcState = Pump2_PC;
    #else
        #if (CY_PSOC4_4200L)
            /* Save the regulator state and put the PHY into suspend mode */
            Pump2_backup.usbState = Pump2_CR1_REG;
            Pump2_USB_POWER_REG |= Pump2_USBIO_ENTER_SLEEP;
            Pump2_CR1_REG &= Pump2_USBIO_CR1_OFF;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(Pump2__SIO)
        Pump2_backup.sioState = Pump2_SIO_REG;
        /* SIO requires unregulated output buffer and single ended input buffer */
        Pump2_SIO_REG &= (uint32)(~Pump2_SIO_LPM_MASK);
    #endif  
}


/*******************************************************************************
* Function Name: Pump2_Wakeup
****************************************************************************//**
*
* \brief Restores the pin configuration that was saved during Pin_Sleep().
*
* For USBIO pins, the wakeup is only triggered for falling edge interrupts.
*
* <b>Note</b> This function is available in PSoC 4 only.
*
* \return 
*  None
*  
* \funcusage
*  Refer to Pump2_Sleep() for an example usage.
*******************************************************************************/
void Pump2_Wakeup(void)
{
    #if defined(Pump2__PC)
        Pump2_PC = Pump2_backup.pcState;
    #else
        #if (CY_PSOC4_4200L)
            /* Restore the regulator state and come out of suspend mode */
            Pump2_USB_POWER_REG &= Pump2_USBIO_EXIT_SLEEP_PH1;
            Pump2_CR1_REG = Pump2_backup.usbState;
            Pump2_USB_POWER_REG &= Pump2_USBIO_EXIT_SLEEP_PH2;
        #endif
    #endif
    #if defined(CYIPBLOCK_m0s8ioss_VERSION) && defined(Pump2__SIO)
        Pump2_SIO_REG = Pump2_backup.sioState;
    #endif
}


/* [] END OF FILE */
