/*******************************************************************************
* File Name: mux_setting_PM.c
* Version 1.80
*
* Description:
*  This file contains the setup, control, and status commands to support 
*  the component operation in the low power mode. 
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "mux_setting.h"

/* Check for removal by optimization */
#if !defined(mux_setting_Sync_ctrl_reg__REMOVED)

static mux_setting_BACKUP_STRUCT  mux_setting_backup = {0u};

    
/*******************************************************************************
* Function Name: mux_setting_SaveConfig
********************************************************************************
*
* Summary:
*  Saves the control register value.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void mux_setting_SaveConfig(void) 
{
    mux_setting_backup.controlState = mux_setting_Control;
}


/*******************************************************************************
* Function Name: mux_setting_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the control register value.
*
* Parameters:
*  None
*
* Return:
*  None
*
*
*******************************************************************************/
void mux_setting_RestoreConfig(void) 
{
     mux_setting_Control = mux_setting_backup.controlState;
}


/*******************************************************************************
* Function Name: mux_setting_Sleep
********************************************************************************
*
* Summary:
*  Prepares the component for entering the low power mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void mux_setting_Sleep(void) 
{
    mux_setting_SaveConfig();
}


/*******************************************************************************
* Function Name: mux_setting_Wakeup
********************************************************************************
*
* Summary:
*  Restores the component after waking up from the low power mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void mux_setting_Wakeup(void)  
{
    mux_setting_RestoreConfig();
}

#endif /* End check for removal by optimization */


/* [] END OF FILE */
