/*******************************************************************************
* File Name: start_timer_PM.c
* Version 1.80
*
* Description:
*  This file contains the setup, control, and status commands to support 
*  the component operation in the low power mode. 
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "start_timer.h"

/* Check for removal by optimization */
#if !defined(start_timer_Sync_ctrl_reg__REMOVED)

static start_timer_BACKUP_STRUCT  start_timer_backup = {0u};

    
/*******************************************************************************
* Function Name: start_timer_SaveConfig
********************************************************************************
*
* Summary:
*  Saves the control register value.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void start_timer_SaveConfig(void) 
{
    start_timer_backup.controlState = start_timer_Control;
}


/*******************************************************************************
* Function Name: start_timer_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the control register value.
*
* Parameters:
*  None
*
* Return:
*  None
*
*
*******************************************************************************/
void start_timer_RestoreConfig(void) 
{
     start_timer_Control = start_timer_backup.controlState;
}


/*******************************************************************************
* Function Name: start_timer_Sleep
********************************************************************************
*
* Summary:
*  Prepares the component for entering the low power mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void start_timer_Sleep(void) 
{
    start_timer_SaveConfig();
}


/*******************************************************************************
* Function Name: start_timer_Wakeup
********************************************************************************
*
* Summary:
*  Restores the component after waking up from the low power mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void start_timer_Wakeup(void)  
{
    start_timer_RestoreConfig();
}

#endif /* End check for removal by optimization */


/* [] END OF FILE */
