/*******************************************************************************
* File Name: switch_pump.h
* Version 2.70
*
*  Description:
*     Contains the function prototypes and constants available to the timer
*     user module.
*
*   Note:
*     None
*
********************************************************************************
* Copyright 2008-2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
********************************************************************************/

#if !defined(CY_Timer_v2_60_switch_pump_H)
#define CY_Timer_v2_60_switch_pump_H

#include "cytypes.h"
#include "cyfitter.h"
#include "CyLib.h" /* For CyEnterCriticalSection() and CyExitCriticalSection() functions */

extern uint8 switch_pump_initVar;

/* Check to see if required defines such as CY_PSOC5LP are available */
/* They are defined starting with cy_boot v3.0 */
#if !defined (CY_PSOC5LP)
    #error Component Timer_v2_70 requires cy_boot v3.0 or later
#endif /* (CY_ PSOC5LP) */


/**************************************
*           Parameter Defaults
**************************************/

#define switch_pump_Resolution                 16u
#define switch_pump_UsingFixedFunction         0u
#define switch_pump_UsingHWCaptureCounter      0u
#define switch_pump_SoftwareCaptureMode        0u
#define switch_pump_SoftwareTriggerMode        0u
#define switch_pump_UsingHWEnable              0u
#define switch_pump_EnableTriggerMode          0u
#define switch_pump_InterruptOnCaptureCount    0u
#define switch_pump_RunModeUsed                1u
#define switch_pump_ControlRegRemoved          0u

#if defined(switch_pump_TimerUDB_sCTRLReg_SyncCtl_ctrlreg__CONTROL_REG)
    #define switch_pump_UDB_CONTROL_REG_REMOVED            (0u)
#elif  (switch_pump_UsingFixedFunction)
    #define switch_pump_UDB_CONTROL_REG_REMOVED            (0u)
#else 
    #define switch_pump_UDB_CONTROL_REG_REMOVED            (1u)
#endif /* End switch_pump_TimerUDB_sCTRLReg_SyncCtl_ctrlreg__CONTROL_REG */


/***************************************
*       Type defines
***************************************/


/**************************************************************************
 * Sleep Wakeup Backup structure for Timer Component
 *************************************************************************/
typedef struct
{
    uint8 TimerEnableState;
    #if(!switch_pump_UsingFixedFunction)

        uint16 TimerUdb;
        uint8 InterruptMaskValue;
        #if (switch_pump_UsingHWCaptureCounter)
            uint8 TimerCaptureCounter;
        #endif /* variable declarations for backing up non retention registers in CY_UDB_V1 */

        #if (!switch_pump_UDB_CONTROL_REG_REMOVED)
            uint8 TimerControlRegister;
        #endif /* variable declaration for backing up enable state of the Timer */
    #endif /* define backup variables only for UDB implementation. Fixed function registers are all retention */

}switch_pump_backupStruct;


/***************************************
*       Function Prototypes
***************************************/

void    switch_pump_Start(void) ;
void    switch_pump_Stop(void) ;

void    switch_pump_SetInterruptMode(uint8 interruptMode) ;
uint8   switch_pump_ReadStatusRegister(void) ;
/* Deprecated function. Do not use this in future. Retained for backward compatibility */
#define switch_pump_GetInterruptSource() switch_pump_ReadStatusRegister()

#if(!switch_pump_UDB_CONTROL_REG_REMOVED)
    uint8   switch_pump_ReadControlRegister(void) ;
    void    switch_pump_WriteControlRegister(uint8 control) ;
#endif /* (!switch_pump_UDB_CONTROL_REG_REMOVED) */

uint16  switch_pump_ReadPeriod(void) ;
void    switch_pump_WritePeriod(uint16 period) ;
uint16  switch_pump_ReadCounter(void) ;
void    switch_pump_WriteCounter(uint16 counter) ;
uint16  switch_pump_ReadCapture(void) ;
void    switch_pump_SoftwareCapture(void) ;

#if(!switch_pump_UsingFixedFunction) /* UDB Prototypes */
    #if (switch_pump_SoftwareCaptureMode)
        void    switch_pump_SetCaptureMode(uint8 captureMode) ;
    #endif /* (!switch_pump_UsingFixedFunction) */

    #if (switch_pump_SoftwareTriggerMode)
        void    switch_pump_SetTriggerMode(uint8 triggerMode) ;
    #endif /* (switch_pump_SoftwareTriggerMode) */

    #if (switch_pump_EnableTriggerMode)
        void    switch_pump_EnableTrigger(void) ;
        void    switch_pump_DisableTrigger(void) ;
    #endif /* (switch_pump_EnableTriggerMode) */


    #if(switch_pump_InterruptOnCaptureCount)
        void    switch_pump_SetInterruptCount(uint8 interruptCount) ;
    #endif /* (switch_pump_InterruptOnCaptureCount) */

    #if (switch_pump_UsingHWCaptureCounter)
        void    switch_pump_SetCaptureCount(uint8 captureCount) ;
        uint8   switch_pump_ReadCaptureCount(void) ;
    #endif /* (switch_pump_UsingHWCaptureCounter) */

    void switch_pump_ClearFIFO(void) ;
#endif /* UDB Prototypes */

/* Sleep Retention APIs */
void switch_pump_Init(void)          ;
void switch_pump_Enable(void)        ;
void switch_pump_SaveConfig(void)    ;
void switch_pump_RestoreConfig(void) ;
void switch_pump_Sleep(void)         ;
void switch_pump_Wakeup(void)        ;


/***************************************
*   Enumerated Types and Parameters
***************************************/

/* Enumerated Type B_Timer__CaptureModes, Used in Capture Mode */
#define switch_pump__B_TIMER__CM_NONE 0
#define switch_pump__B_TIMER__CM_RISINGEDGE 1
#define switch_pump__B_TIMER__CM_FALLINGEDGE 2
#define switch_pump__B_TIMER__CM_EITHEREDGE 3
#define switch_pump__B_TIMER__CM_SOFTWARE 4



/* Enumerated Type B_Timer__TriggerModes, Used in Trigger Mode */
#define switch_pump__B_TIMER__TM_NONE 0x00u
#define switch_pump__B_TIMER__TM_RISINGEDGE 0x04u
#define switch_pump__B_TIMER__TM_FALLINGEDGE 0x08u
#define switch_pump__B_TIMER__TM_EITHEREDGE 0x0Cu
#define switch_pump__B_TIMER__TM_SOFTWARE 0x10u


/***************************************
*    Initialial Parameter Constants
***************************************/

#define switch_pump_INIT_PERIOD             4999u
#define switch_pump_INIT_CAPTURE_MODE       ((uint8)((uint8)0u << switch_pump_CTRL_CAP_MODE_SHIFT))
#define switch_pump_INIT_TRIGGER_MODE       ((uint8)((uint8)0u << switch_pump_CTRL_TRIG_MODE_SHIFT))
#if (switch_pump_UsingFixedFunction)
    #define switch_pump_INIT_INTERRUPT_MODE (((uint8)((uint8)1u << switch_pump_STATUS_TC_INT_MASK_SHIFT)) | \
                                                  ((uint8)((uint8)0 << switch_pump_STATUS_CAPTURE_INT_MASK_SHIFT)))
#else
    #define switch_pump_INIT_INTERRUPT_MODE (((uint8)((uint8)1u << switch_pump_STATUS_TC_INT_MASK_SHIFT)) | \
                                                 ((uint8)((uint8)0 << switch_pump_STATUS_CAPTURE_INT_MASK_SHIFT)) | \
                                                 ((uint8)((uint8)0 << switch_pump_STATUS_FIFOFULL_INT_MASK_SHIFT)))
#endif /* (switch_pump_UsingFixedFunction) */
#define switch_pump_INIT_CAPTURE_COUNT      (2u)
#define switch_pump_INIT_INT_CAPTURE_COUNT  ((uint8)((uint8)(1u - 1u) << switch_pump_CTRL_INTCNT_SHIFT))


/***************************************
*           Registers
***************************************/

#if (switch_pump_UsingFixedFunction) /* Implementation Specific Registers and Register Constants */


    /***************************************
    *    Fixed Function Registers
    ***************************************/

    #define switch_pump_STATUS         (*(reg8 *) switch_pump_TimerHW__SR0 )
    /* In Fixed Function Block Status and Mask are the same register */
    #define switch_pump_STATUS_MASK    (*(reg8 *) switch_pump_TimerHW__SR0 )
    #define switch_pump_CONTROL        (*(reg8 *) switch_pump_TimerHW__CFG0)
    #define switch_pump_CONTROL2       (*(reg8 *) switch_pump_TimerHW__CFG1)
    #define switch_pump_CONTROL2_PTR   ( (reg8 *) switch_pump_TimerHW__CFG1)
    #define switch_pump_RT1            (*(reg8 *) switch_pump_TimerHW__RT1)
    #define switch_pump_RT1_PTR        ( (reg8 *) switch_pump_TimerHW__RT1)

    #if (CY_PSOC3 || CY_PSOC5LP)
        #define switch_pump_CONTROL3       (*(reg8 *) switch_pump_TimerHW__CFG2)
        #define switch_pump_CONTROL3_PTR   ( (reg8 *) switch_pump_TimerHW__CFG2)
    #endif /* (CY_PSOC3 || CY_PSOC5LP) */
    #define switch_pump_GLOBAL_ENABLE  (*(reg8 *) switch_pump_TimerHW__PM_ACT_CFG)
    #define switch_pump_GLOBAL_STBY_ENABLE  (*(reg8 *) switch_pump_TimerHW__PM_STBY_CFG)

    #define switch_pump_CAPTURE_LSB         (* (reg16 *) switch_pump_TimerHW__CAP0 )
    #define switch_pump_CAPTURE_LSB_PTR       ((reg16 *) switch_pump_TimerHW__CAP0 )
    #define switch_pump_PERIOD_LSB          (* (reg16 *) switch_pump_TimerHW__PER0 )
    #define switch_pump_PERIOD_LSB_PTR        ((reg16 *) switch_pump_TimerHW__PER0 )
    #define switch_pump_COUNTER_LSB         (* (reg16 *) switch_pump_TimerHW__CNT_CMP0 )
    #define switch_pump_COUNTER_LSB_PTR       ((reg16 *) switch_pump_TimerHW__CNT_CMP0 )


    /***************************************
    *    Register Constants
    ***************************************/

    /* Fixed Function Block Chosen */
    #define switch_pump_BLOCK_EN_MASK                     switch_pump_TimerHW__PM_ACT_MSK
    #define switch_pump_BLOCK_STBY_EN_MASK                switch_pump_TimerHW__PM_STBY_MSK

    /* Control Register Bit Locations */
    /* Interrupt Count - Not valid for Fixed Function Block */
    #define switch_pump_CTRL_INTCNT_SHIFT                  0x00u
    /* Trigger Polarity - Not valid for Fixed Function Block */
    #define switch_pump_CTRL_TRIG_MODE_SHIFT               0x00u
    /* Trigger Enable - Not valid for Fixed Function Block */
    #define switch_pump_CTRL_TRIG_EN_SHIFT                 0x00u
    /* Capture Polarity - Not valid for Fixed Function Block */
    #define switch_pump_CTRL_CAP_MODE_SHIFT                0x00u
    /* Timer Enable - As defined in Register Map, part of TMRX_CFG0 register */
    #define switch_pump_CTRL_ENABLE_SHIFT                  0x00u

    /* Control Register Bit Masks */
    #define switch_pump_CTRL_ENABLE                        ((uint8)((uint8)0x01u << switch_pump_CTRL_ENABLE_SHIFT))

    /* Control2 Register Bit Masks */
    /* As defined in Register Map, Part of the TMRX_CFG1 register */
    #define switch_pump_CTRL2_IRQ_SEL_SHIFT                 0x00u
    #define switch_pump_CTRL2_IRQ_SEL                      ((uint8)((uint8)0x01u << switch_pump_CTRL2_IRQ_SEL_SHIFT))

    #if (CY_PSOC5A)
        /* Use CFG1 Mode bits to set run mode */
        /* As defined by Verilog Implementation */
        #define switch_pump_CTRL_MODE_SHIFT                 0x01u
        #define switch_pump_CTRL_MODE_MASK                 ((uint8)((uint8)0x07u << switch_pump_CTRL_MODE_SHIFT))
    #endif /* (CY_PSOC5A) */
    #if (CY_PSOC3 || CY_PSOC5LP)
        /* Control3 Register Bit Locations */
        #define switch_pump_CTRL_RCOD_SHIFT        0x02u
        #define switch_pump_CTRL_ENBL_SHIFT        0x00u
        #define switch_pump_CTRL_MODE_SHIFT        0x00u

        /* Control3 Register Bit Masks */
        #define switch_pump_CTRL_RCOD_MASK  ((uint8)((uint8)0x03u << switch_pump_CTRL_RCOD_SHIFT)) /* ROD and COD bit masks */
        #define switch_pump_CTRL_ENBL_MASK  ((uint8)((uint8)0x80u << switch_pump_CTRL_ENBL_SHIFT)) /* HW_EN bit mask */
        #define switch_pump_CTRL_MODE_MASK  ((uint8)((uint8)0x03u << switch_pump_CTRL_MODE_SHIFT)) /* Run mode bit mask */

        #define switch_pump_CTRL_RCOD       ((uint8)((uint8)0x03u << switch_pump_CTRL_RCOD_SHIFT))
        #define switch_pump_CTRL_ENBL       ((uint8)((uint8)0x80u << switch_pump_CTRL_ENBL_SHIFT))
    #endif /* (CY_PSOC3 || CY_PSOC5LP) */

    /*RT1 Synch Constants: Applicable for PSoC3 and PSoC5LP */
    #define switch_pump_RT1_SHIFT                       0x04u
    /* Sync TC and CMP bit masks */
    #define switch_pump_RT1_MASK                        ((uint8)((uint8)0x03u << switch_pump_RT1_SHIFT))
    #define switch_pump_SYNC                            ((uint8)((uint8)0x03u << switch_pump_RT1_SHIFT))
    #define switch_pump_SYNCDSI_SHIFT                   0x00u
    /* Sync all DSI inputs with Mask  */
    #define switch_pump_SYNCDSI_MASK                    ((uint8)((uint8)0x0Fu << switch_pump_SYNCDSI_SHIFT))
    /* Sync all DSI inputs */
    #define switch_pump_SYNCDSI_EN                      ((uint8)((uint8)0x0Fu << switch_pump_SYNCDSI_SHIFT))

    #define switch_pump_CTRL_MODE_PULSEWIDTH            ((uint8)((uint8)0x01u << switch_pump_CTRL_MODE_SHIFT))
    #define switch_pump_CTRL_MODE_PERIOD                ((uint8)((uint8)0x02u << switch_pump_CTRL_MODE_SHIFT))
    #define switch_pump_CTRL_MODE_CONTINUOUS            ((uint8)((uint8)0x00u << switch_pump_CTRL_MODE_SHIFT))

    /* Status Register Bit Locations */
    /* As defined in Register Map, part of TMRX_SR0 register */
    #define switch_pump_STATUS_TC_SHIFT                 0x07u
    /* As defined in Register Map, part of TMRX_SR0 register, Shared with Compare Status */
    #define switch_pump_STATUS_CAPTURE_SHIFT            0x06u
    /* As defined in Register Map, part of TMRX_SR0 register */
    #define switch_pump_STATUS_TC_INT_MASK_SHIFT        (switch_pump_STATUS_TC_SHIFT - 0x04u)
    /* As defined in Register Map, part of TMRX_SR0 register, Shared with Compare Status */
    #define switch_pump_STATUS_CAPTURE_INT_MASK_SHIFT   (switch_pump_STATUS_CAPTURE_SHIFT - 0x04u)

    /* Status Register Bit Masks */
    #define switch_pump_STATUS_TC                       ((uint8)((uint8)0x01u << switch_pump_STATUS_TC_SHIFT))
    #define switch_pump_STATUS_CAPTURE                  ((uint8)((uint8)0x01u << switch_pump_STATUS_CAPTURE_SHIFT))
    /* Interrupt Enable Bit-Mask for interrupt on TC */
    #define switch_pump_STATUS_TC_INT_MASK              ((uint8)((uint8)0x01u << switch_pump_STATUS_TC_INT_MASK_SHIFT))
    /* Interrupt Enable Bit-Mask for interrupt on Capture */
    #define switch_pump_STATUS_CAPTURE_INT_MASK         ((uint8)((uint8)0x01u << switch_pump_STATUS_CAPTURE_INT_MASK_SHIFT))

#else   /* UDB Registers and Register Constants */


    /***************************************
    *           UDB Registers
    ***************************************/

    #define switch_pump_STATUS              (* (reg8 *) switch_pump_TimerUDB_rstSts_stsreg__STATUS_REG )
    #define switch_pump_STATUS_MASK         (* (reg8 *) switch_pump_TimerUDB_rstSts_stsreg__MASK_REG)
    #define switch_pump_STATUS_AUX_CTRL     (* (reg8 *) switch_pump_TimerUDB_rstSts_stsreg__STATUS_AUX_CTL_REG)
    #define switch_pump_CONTROL             (* (reg8 *) switch_pump_TimerUDB_sCTRLReg_SyncCtl_ctrlreg__CONTROL_REG )
    
    #if(switch_pump_Resolution <= 8u) /* 8-bit Timer */
        #define switch_pump_CAPTURE_LSB         (* (reg8 *) switch_pump_TimerUDB_sT16_timerdp_u0__F0_REG )
        #define switch_pump_CAPTURE_LSB_PTR       ((reg8 *) switch_pump_TimerUDB_sT16_timerdp_u0__F0_REG )
        #define switch_pump_PERIOD_LSB          (* (reg8 *) switch_pump_TimerUDB_sT16_timerdp_u0__D0_REG )
        #define switch_pump_PERIOD_LSB_PTR        ((reg8 *) switch_pump_TimerUDB_sT16_timerdp_u0__D0_REG )
        #define switch_pump_COUNTER_LSB         (* (reg8 *) switch_pump_TimerUDB_sT16_timerdp_u0__A0_REG )
        #define switch_pump_COUNTER_LSB_PTR       ((reg8 *) switch_pump_TimerUDB_sT16_timerdp_u0__A0_REG )
    #elif(switch_pump_Resolution <= 16u) /* 8-bit Timer */
        #if(CY_PSOC3) /* 8-bit addres space */
            #define switch_pump_CAPTURE_LSB         (* (reg16 *) switch_pump_TimerUDB_sT16_timerdp_u0__F0_REG )
            #define switch_pump_CAPTURE_LSB_PTR       ((reg16 *) switch_pump_TimerUDB_sT16_timerdp_u0__F0_REG )
            #define switch_pump_PERIOD_LSB          (* (reg16 *) switch_pump_TimerUDB_sT16_timerdp_u0__D0_REG )
            #define switch_pump_PERIOD_LSB_PTR        ((reg16 *) switch_pump_TimerUDB_sT16_timerdp_u0__D0_REG )
            #define switch_pump_COUNTER_LSB         (* (reg16 *) switch_pump_TimerUDB_sT16_timerdp_u0__A0_REG )
            #define switch_pump_COUNTER_LSB_PTR       ((reg16 *) switch_pump_TimerUDB_sT16_timerdp_u0__A0_REG )
        #else /* 16-bit address space */
            #define switch_pump_CAPTURE_LSB         (* (reg16 *) switch_pump_TimerUDB_sT16_timerdp_u0__16BIT_F0_REG )
            #define switch_pump_CAPTURE_LSB_PTR       ((reg16 *) switch_pump_TimerUDB_sT16_timerdp_u0__16BIT_F0_REG )
            #define switch_pump_PERIOD_LSB          (* (reg16 *) switch_pump_TimerUDB_sT16_timerdp_u0__16BIT_D0_REG )
            #define switch_pump_PERIOD_LSB_PTR        ((reg16 *) switch_pump_TimerUDB_sT16_timerdp_u0__16BIT_D0_REG )
            #define switch_pump_COUNTER_LSB         (* (reg16 *) switch_pump_TimerUDB_sT16_timerdp_u0__16BIT_A0_REG )
            #define switch_pump_COUNTER_LSB_PTR       ((reg16 *) switch_pump_TimerUDB_sT16_timerdp_u0__16BIT_A0_REG )
        #endif /* CY_PSOC3 */
    #elif(switch_pump_Resolution <= 24u)/* 24-bit Timer */
        #define switch_pump_CAPTURE_LSB         (* (reg32 *) switch_pump_TimerUDB_sT16_timerdp_u0__F0_REG )
        #define switch_pump_CAPTURE_LSB_PTR       ((reg32 *) switch_pump_TimerUDB_sT16_timerdp_u0__F0_REG )
        #define switch_pump_PERIOD_LSB          (* (reg32 *) switch_pump_TimerUDB_sT16_timerdp_u0__D0_REG )
        #define switch_pump_PERIOD_LSB_PTR        ((reg32 *) switch_pump_TimerUDB_sT16_timerdp_u0__D0_REG )
        #define switch_pump_COUNTER_LSB         (* (reg32 *) switch_pump_TimerUDB_sT16_timerdp_u0__A0_REG )
        #define switch_pump_COUNTER_LSB_PTR       ((reg32 *) switch_pump_TimerUDB_sT16_timerdp_u0__A0_REG )
    #else /* 32-bit Timer */
        #if(CY_PSOC3 || CY_PSOC5) /* 8-bit address space */
            #define switch_pump_CAPTURE_LSB         (* (reg32 *) switch_pump_TimerUDB_sT16_timerdp_u0__F0_REG )
            #define switch_pump_CAPTURE_LSB_PTR       ((reg32 *) switch_pump_TimerUDB_sT16_timerdp_u0__F0_REG )
            #define switch_pump_PERIOD_LSB          (* (reg32 *) switch_pump_TimerUDB_sT16_timerdp_u0__D0_REG )
            #define switch_pump_PERIOD_LSB_PTR        ((reg32 *) switch_pump_TimerUDB_sT16_timerdp_u0__D0_REG )
            #define switch_pump_COUNTER_LSB         (* (reg32 *) switch_pump_TimerUDB_sT16_timerdp_u0__A0_REG )
            #define switch_pump_COUNTER_LSB_PTR       ((reg32 *) switch_pump_TimerUDB_sT16_timerdp_u0__A0_REG )
        #else /* 32-bit address space */
            #define switch_pump_CAPTURE_LSB         (* (reg32 *) switch_pump_TimerUDB_sT16_timerdp_u0__32BIT_F0_REG )
            #define switch_pump_CAPTURE_LSB_PTR       ((reg32 *) switch_pump_TimerUDB_sT16_timerdp_u0__32BIT_F0_REG )
            #define switch_pump_PERIOD_LSB          (* (reg32 *) switch_pump_TimerUDB_sT16_timerdp_u0__32BIT_D0_REG )
            #define switch_pump_PERIOD_LSB_PTR        ((reg32 *) switch_pump_TimerUDB_sT16_timerdp_u0__32BIT_D0_REG )
            #define switch_pump_COUNTER_LSB         (* (reg32 *) switch_pump_TimerUDB_sT16_timerdp_u0__32BIT_A0_REG )
            #define switch_pump_COUNTER_LSB_PTR       ((reg32 *) switch_pump_TimerUDB_sT16_timerdp_u0__32BIT_A0_REG )
        #endif /* CY_PSOC3 || CY_PSOC5 */ 
    #endif

    #define switch_pump_COUNTER_LSB_PTR_8BIT       ((reg8 *) switch_pump_TimerUDB_sT16_timerdp_u0__A0_REG )
    
    #if (switch_pump_UsingHWCaptureCounter)
        #define switch_pump_CAP_COUNT              (*(reg8 *) switch_pump_TimerUDB_sCapCount_counter__PERIOD_REG )
        #define switch_pump_CAP_COUNT_PTR          ( (reg8 *) switch_pump_TimerUDB_sCapCount_counter__PERIOD_REG )
        #define switch_pump_CAPTURE_COUNT_CTRL     (*(reg8 *) switch_pump_TimerUDB_sCapCount_counter__CONTROL_AUX_CTL_REG )
        #define switch_pump_CAPTURE_COUNT_CTRL_PTR ( (reg8 *) switch_pump_TimerUDB_sCapCount_counter__CONTROL_AUX_CTL_REG )
    #endif /* (switch_pump_UsingHWCaptureCounter) */


    /***************************************
    *       Register Constants
    ***************************************/

    /* Control Register Bit Locations */
    #define switch_pump_CTRL_INTCNT_SHIFT              0x00u       /* As defined by Verilog Implementation */
    #define switch_pump_CTRL_TRIG_MODE_SHIFT           0x02u       /* As defined by Verilog Implementation */
    #define switch_pump_CTRL_TRIG_EN_SHIFT             0x04u       /* As defined by Verilog Implementation */
    #define switch_pump_CTRL_CAP_MODE_SHIFT            0x05u       /* As defined by Verilog Implementation */
    #define switch_pump_CTRL_ENABLE_SHIFT              0x07u       /* As defined by Verilog Implementation */

    /* Control Register Bit Masks */
    #define switch_pump_CTRL_INTCNT_MASK               ((uint8)((uint8)0x03u << switch_pump_CTRL_INTCNT_SHIFT))
    #define switch_pump_CTRL_TRIG_MODE_MASK            ((uint8)((uint8)0x03u << switch_pump_CTRL_TRIG_MODE_SHIFT))
    #define switch_pump_CTRL_TRIG_EN                   ((uint8)((uint8)0x01u << switch_pump_CTRL_TRIG_EN_SHIFT))
    #define switch_pump_CTRL_CAP_MODE_MASK             ((uint8)((uint8)0x03u << switch_pump_CTRL_CAP_MODE_SHIFT))
    #define switch_pump_CTRL_ENABLE                    ((uint8)((uint8)0x01u << switch_pump_CTRL_ENABLE_SHIFT))

    /* Bit Counter (7-bit) Control Register Bit Definitions */
    /* As defined by the Register map for the AUX Control Register */
    #define switch_pump_CNTR_ENABLE                    0x20u

    /* Status Register Bit Locations */
    #define switch_pump_STATUS_TC_SHIFT                0x00u  /* As defined by Verilog Implementation */
    #define switch_pump_STATUS_CAPTURE_SHIFT           0x01u  /* As defined by Verilog Implementation */
    #define switch_pump_STATUS_TC_INT_MASK_SHIFT       switch_pump_STATUS_TC_SHIFT
    #define switch_pump_STATUS_CAPTURE_INT_MASK_SHIFT  switch_pump_STATUS_CAPTURE_SHIFT
    #define switch_pump_STATUS_FIFOFULL_SHIFT          0x02u  /* As defined by Verilog Implementation */
    #define switch_pump_STATUS_FIFONEMP_SHIFT          0x03u  /* As defined by Verilog Implementation */
    #define switch_pump_STATUS_FIFOFULL_INT_MASK_SHIFT switch_pump_STATUS_FIFOFULL_SHIFT

    /* Status Register Bit Masks */
    /* Sticky TC Event Bit-Mask */
    #define switch_pump_STATUS_TC                      ((uint8)((uint8)0x01u << switch_pump_STATUS_TC_SHIFT))
    /* Sticky Capture Event Bit-Mask */
    #define switch_pump_STATUS_CAPTURE                 ((uint8)((uint8)0x01u << switch_pump_STATUS_CAPTURE_SHIFT))
    /* Interrupt Enable Bit-Mask */
    #define switch_pump_STATUS_TC_INT_MASK             ((uint8)((uint8)0x01u << switch_pump_STATUS_TC_SHIFT))
    /* Interrupt Enable Bit-Mask */
    #define switch_pump_STATUS_CAPTURE_INT_MASK        ((uint8)((uint8)0x01u << switch_pump_STATUS_CAPTURE_SHIFT))
    /* NOT-Sticky FIFO Full Bit-Mask */
    #define switch_pump_STATUS_FIFOFULL                ((uint8)((uint8)0x01u << switch_pump_STATUS_FIFOFULL_SHIFT))
    /* NOT-Sticky FIFO Not Empty Bit-Mask */
    #define switch_pump_STATUS_FIFONEMP                ((uint8)((uint8)0x01u << switch_pump_STATUS_FIFONEMP_SHIFT))
    /* Interrupt Enable Bit-Mask */
    #define switch_pump_STATUS_FIFOFULL_INT_MASK       ((uint8)((uint8)0x01u << switch_pump_STATUS_FIFOFULL_SHIFT))

    #define switch_pump_STATUS_ACTL_INT_EN             0x10u   /* As defined for the ACTL Register */

    /* Datapath Auxillary Control Register definitions */
    #define switch_pump_AUX_CTRL_FIFO0_CLR             0x01u   /* As defined by Register map */
    #define switch_pump_AUX_CTRL_FIFO1_CLR             0x02u   /* As defined by Register map */
    #define switch_pump_AUX_CTRL_FIFO0_LVL             0x04u   /* As defined by Register map */
    #define switch_pump_AUX_CTRL_FIFO1_LVL             0x08u   /* As defined by Register map */
    #define switch_pump_STATUS_ACTL_INT_EN_MASK        0x10u   /* As defined for the ACTL Register */

#endif /* Implementation Specific Registers and Register Constants */

#endif  /* CY_Timer_v2_30_switch_pump_H */


/* [] END OF FILE */
